class PurseType(object):
    TYPE_WEBMONEY = 'WebMoney'
    TYPE_YANDEX_MONEY = 'Yandex'
    TYPE_QIWI = 'QIWI'
    TYPE_ZALEY_CASH = 'ZaleyCash'
    TYPE_RU_CARD = 'RUCARD'
    TYPE_UKR_CARD = 'UKRCARD'
    TYPE_WORLD_CARD = 'WORLDCARD'
    TYPE_CIS_CARD = 'SNGCARD'
    TYPE_MTS = 'MTS'
    TYPE_BEELINE = 'Beeline'
    TYPE_MEGAFON = 'MegaFon'
    TYPE_TELE2 = 'Tele2'
    TYPE_CAPITALIST = 'Capitalist'


class Currency(object):
    RUB = 'RUB'
    USD = 'USD'
    EURO = 'EUR'



import logging

from django.core.management import BaseCommand
from django.db import transaction

from django_master_pay import settings
from django_master_pay.api import MasterPayApi, MasterPayApiException
from django_master_pay.models import Payment


class Command(BaseCommand):

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.logger = logging.getLogger(settings.MASTER_PAY_SETTINGS['logger_name'])
        self.callback = settings.MASTER_PAY_SETTINGS['callback']
        self.api = MasterPayApi()

    def handle(self, *args, **options):
        self.logger.info('START')

        payments = Payment.objects.filter(
            status__in=Payment.WORK_STATUSES
        )

        for payment in payments:
            self.process_payment(payment)

        self.logger.info('END')

    def process_payment(self, payment):
        self.logger.info('Process payment #{}'.format(payment.id))

        try:
            payment_data = self.api.get_payment(
                payment_id=payment.master_pay_id,
                partner_id=payment.partner_id
            )
        except MasterPayApiException as e:
            self.logger.warning("\t{}".format(str(e)))
        else:
            if payment_data['status'] != payment.status:
                self.logger.info('\tNew status: OLD={} NEW={}'.format(Payment.status_display(payment.status), Payment.status_display(payment_data['status'])))
                with transaction.atomic():
                    payment.status = payment_data['status']
                    payment.record_data = payment_data
                    payment.error = payment_data['auto_payment_error']
                    payment.save(update_fields=['status', 'record_data', 'error'])
                    self.callback(**payment_data)



from django.db import transaction

from django_master_pay.api import MasterPayApi
from django_master_pay.settings import MASTER_PAY_SETTINGS


def create_payment(amount, currency, purse_type, purse_number, external_id, partner_id=MASTER_PAY_SETTINGS['default_partner_id']):
    from django_master_pay.models import Payment
    api = MasterPayApi()

    with transaction.atomic():
        payment_data = api.create_payment(amount, purse_type, currency, purse_number, external_id, partner_id=partner_id)
        payment = Payment(
            external_id=external_id,
            master_pay_id=payment_data['id'],
            record_data=payment_data,
            partner_id=partner_id
        )
        payment.save()
        return payment



from django.contrib import admin

from django_master_pay.models import Payment


@admin.register(Payment)
class PaymentAdmin(admin.ModelAdmin):
    list_display = ('id', 'external_id', 'master_pay_id', 'partner_id', 'status', 'created_at', 'updated_at')
    list_filter = ('status', 'created_at', 'updated_at')
    search_fields = ('id', 'external_id', 'master_pay_id')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False



from django.contrib.postgres.fields import JSONField
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class Payment(models.Model):
    STATUS_WAIT = 0
    STATUS_PAID = 1
    STATUS_PAID_PERCENT = 2
    STATUS_CANCEL = 3
    STATUS_COMPLETE = 4
    STATUS_ERROR = 5

    STATUS_CHOICES = (
        (STATUS_WAIT, _("Ожидает")),
        (STATUS_PAID, _("Выплачивается")),
        (STATUS_PAID_PERCENT, _("Выплачен частично")),
        (STATUS_ERROR, _("Ошибка")),
        (STATUS_COMPLETE, _("Выплачен")),
        (STATUS_CANCEL, _("Отменен пользователем"))
    )

    WORK_STATUSES = (STATUS_WAIT, STATUS_PAID, STATUS_PAID_PERCENT)

    created_at = models.DateTimeField(_('Создан'), default=timezone.now)
    updated_at = models.DateTimeField(_('Обновлен'), auto_now=True)
    external_id = models.CharField(_("Внешний ключ"), max_length=250)
    master_pay_id = models.CharField(_("Индефикатор MasterPay"), max_length=250)
    partner_id = models.CharField(_("Индефикатор партнера в MasterPay"), max_length=250)
    status = models.PositiveSmallIntegerField(_("Статус"), choices=STATUS_CHOICES, default=STATUS_WAIT)
    error = models.TextField(_('Ошибка автоматической выплаты'), blank=True, null=True)
    record_data = JSONField(_("RAW data"))

    class Meta:
        verbose_name = _("Платеж")
        verbose_name_plural = _("Платежи")

    def __str__(self):
        return "#{} external_id={} master_pay_id={} status={}".format(
            self.id, self.external_id, self.master_pay_id, self.status
        )

    @classmethod
    def status_display(cls, status):
        status_dict = dict(cls.STATUS_CHOICES)
        return status_dict[status]



# Generated by Django 2.2.2 on 2021-07-26 10:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_master_pay', '0002_auto_20210721_1229'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payment',
            name='status',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Ожидает'), (1, 'Выплачивается'), (2, 'Выплачен частично'), (5, 'Ошибка'), (4, 'Выплачен'), (3, 'Отменен пользователем')], default=0, max_length=100, verbose_name='Статус'),
        ),
    ]
